﻿using System;
using System.Data;
using System.Configuration;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;

/// <summary>
/// Contains basic methods required
/// for the dynamic-controls to function 
/// properly
/// </summary>
public class BasePage : Page
{
    #region Constants
    //private const string TYPE_CACHE_KEY = "Type_{0}";
    
    // Holds the Session Key to store the added-controls on page
    private const string SESSION_CONTAINERS_KEY = "Controls_In_Container";
    
    // Holds the path to a user control
    private const string PATH_TO_CONTROL = "~/App_Controls/{0}";
    #endregion

    #region Control Methods
    /// <summary>
    /// This method takes as input the PlaceHolder
    /// where to load controls to. This allows you to 
    /// load controls in any container placed on
    /// the page. It simply loops through the specific
    /// container, identified by the passed argument's
    /// ClientID, dictionary and adds every control found
    /// into the passed in place holder.
    /// </summary>
    protected void LoadExistingControls(PlaceHolder phContainer)
    {
        // If there are controls to load
        if (
            (this.Container != null) && 
            (this.Container.Count > 0)
            )
        {
            // If the container hasn't been intialized beofre
            if (!this.Container.ContainsKey(phContainer.ClientID))
                return;
            
            // Clear all previous controls
            phContainer.Controls.Clear();

            // Get every KeyValuePair, extract the UserControl from the Value
            // and add it to the container passed as parameter. 
            foreach (KeyValuePair<string, UserControl> value in this.Container[phContainer.ClientID])
            {
                phContainer.Controls.Add(value.Value);
            }
        }
    }

    /// <summary>
    /// Creates a new instance of a Control defined
    /// by a name. This method takes as input the unique control
    /// key to set a unique ID property
    /// </summary>
    /// <param name="t"></param>
    /// <returns></returns>
    protected UserControl CreateControlInstance(string controlName, string controlKey)
    {
        // Create a new instance of the control
        UserControl newControl =
            (UserControl)LoadControl(string.Format(PATH_TO_CONTROL, controlName));

        // Set the ID of the new control instance
        newControl.ID = string.Format(controlKey, Guid.NewGuid().ToString());

        return newControl;
    }

    /// <summary>
    /// It takes as input the UserControl to add and a reference
    /// to the place holder on the page. It simply adds the passed
    /// in usercontrol and adds it to the container.
    /// </summary>
    /// <param name="c"></param>
    protected void AddControlToContainer(UserControl c, PlaceHolder phContainer)
    {
        // Define a dictionary of containers
        Dictionary<string, Dictionary<string, UserControl>> containers = null;

        // If there are defined containers with usercontrols in the session, get them
        if (Session[SESSION_CONTAINERS_KEY] != null)
            containers = (Dictionary<string, Dictionary<string, UserControl>>)Session[SESSION_CONTAINERS_KEY];
        
        // Seems there are no pre-defined containers containg usercontrols in the Session,
        // create a new dictionary
        if (containers == null)
            containers = new Dictionary<string, Dictionary<string, UserControl>>();

        // If this is the first time we are adding a Usercontrol to a container
        // add a record for this placeholder identified by the place holder's
        // ClientID.
        if (!containers.ContainsKey(phContainer.ClientID))
            containers.Add(phContainer.ClientID, new Dictionary<string, UserControl>());

        // Add to the specified container a new record
        // having as value the ClientID of the UserControl
        // and as value the UserControl itself.
        if (containers[phContainer.ClientID] != null)
            containers[phContainer.ClientID].Add(c.ClientID, c);

        // Update the session variable
        Session[SESSION_CONTAINERS_KEY] = containers;
    }
    #endregion

    #region Control Properties
    /// <summary>
    /// Holds a dictionary of all the containers defined on 
    /// the page that are used as place holders for dynamically
    /// added controls. This property is a dictionary with the 
    /// place holder's ClientID serving as the key, and as value 
    /// a dictionary with the key as a UserControl's ClientID 
    /// and value of the  UserControl itself.
    /// This property checks to see if there are any loaded containers
    /// in the Session variable, if not it creates a new instance
    /// of an empty Dictionary of containers
    /// </summary>
    protected Dictionary<string, Dictionary<string, UserControl>> Container
    {
        get
        {
            // Create a new instance of a dictionary
            // holding as Key the name of the Conatiner
            // and as Value another Dictionary. The inline Dictionary
            // has as Key the ClientID of the UserControl added to the page
            // and as Value the UserControl itself
            Dictionary<string, Dictionary<string, UserControl>> containers =
                new Dictionary<string, Dictionary<string, UserControl>>();

            // If there are containers in the Session variable
            // load them
            if (Session[SESSION_CONTAINERS_KEY] != null)
                containers = (Dictionary<string, Dictionary<string, UserControl>>)Session[SESSION_CONTAINERS_KEY];

            return containers;
        }
    }
    #endregion
}